const mongoose = require('mongoose');
const express = require('express');
const cookieParser = require('cookie-parser');
const fs = require('fs');
const crypto = require('crypto');
const cm = require('./customsessions');

cm.sessions.startCleanup();

const connection_string = 'mongodb://127.0.0.1/responsibilities';

mongoose.connect(connection_string);
mongoose.connection.on('error', () => {
  console.log('There was a problem connecting to mongoDB');
});

/**
 * Specify the schema for the database.
 */
var UserSchema = new mongoose.Schema( {
    username: String,
    salt: Number,
    hash: String,
    favoriteColors: [String]
});
var User = mongoose.model('User', UserSchema);
var ItemSchema = new mongoose.Schema( {
    name: String,
    difficulty: Number,
    stat: String
});
var Item = mongoose.model('Item', ItemSchema);

function authenticate(req, res, next) {
  let c = req.cookies;
  if (c && c.login) {
    let result = cm.sessions.doesUserHaveSession(c.login.username, c.login.sid);
    if (result) {
      next();
      return;
    }
  }
  res.redirect('/account/index.html');
}

/**
 * Initialize the express app and configure with various features 
 * such as JSON parsing, static file serving, etc.
 */
const app = express();
app.use(cookieParser());    
app.use(express.json())
app.use(express.static('public_html'))

app.use(function (req, res, next) {
    let origin = req.headers.origin;
    //res.header("Access-Control-Allow-Origin", "*");
    res.header("Access-Control-Allow-Origin", "http://localhost:3000");
    res.header("Access-Control-Allow-Methods", "GET,HEAD,OPTIONS,POST,PUT");
    res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept, Authorization");
    res.header('Access-Control-Allow-Credentials', true);
    next();
});
        
app.get('/isloggedin', (req, res) => {
  let c = req.cookies;
  if (c && c.login) {
    let result = cm.sessions.doesUserHaveSession(c.login.username, c.login.sid);
    if (result) {
      res.end('yes');
      return;
    }
  }
  res.end('no');
});


app.use('*', (req, res, next) => {
  let c = req.cookies;
  if (c && c.login) {
    if (cm.sessions.doesUserHaveSession(c.login.username, c.login.sid)) {
      cm.sessions.addOrUpdateSession(c.login.username);
    }
  }
  next();
});

/**
 * Get all of the items that correspond with a particular catagory.
 */
app.get('/items/', (req, res) => {
    let p1 = Item.find({}).exec();
    p1.then( (results) => { 
      res.end( JSON.stringify(results) );
    });
    p1.catch( (error) => {
      console.log(error);
      res.end('FAIL');
    });
});

/**
 * This route is for removing an item.
 */
app.get('/remove/item/:id', (req, res) => {
  let id = req.params.id;
  let p = Item.deleteOne({_id: id}).exec();
  p.then((doc) => { 
    res.send('deleted');
  });
  p.catch((doc) => { 
    res.send('failed');
  });
});

/**
 * This route is for creating a new item.
 */
app.post('/create/item/', (req, res) => {
  let newItemToSave = req.body;
  var newItem = new Item(newItemToSave);
  let p1 = newItem.save();
  p1.then( (doc) => { 
    res.end('SAVED SUCCESFULLY');
  });
  p1.catch( (err) => { 
    console.log(err);
    res.end('FAIL');
  });
});

/**
 * This route is for creating a new user account.
 */
app.get('/account/create/:username/:password', (req, res) => {
  let p1 = User.find({username: req.params.username}).exec();
  p1.then( (results) => { 
    if (results.length > 0) {
      res.end('That username is already taken.');
    } else {

      let newSalt = Math.floor((Math.random() * 1000000));
      let toHash = req.params.password + newSalt;
      var hash = crypto.createHash('sha3-256');
      let data = hash.update(toHash, 'utf-8');
      let newHash = data.digest('hex');

      var newUser = new User({ 
        username: req.params.username,
        salt: newSalt,
        hash: newHash
      });
      newUser.save().then( (doc) => { 
          res.end('Created new account!');
        }).catch( (err) => { 
          console.log(err);
          res.end('Failed to create new account.');
        });
    }
  });
  p1.catch( (error) => {
    res.end('Failed to create new account.');
  });
});

/**
 * Process a user login request.
 */
app.post('/account/login/', (req, res) => {
  let u = req.body.username;
  let p = req.body.password;
  console.log(u);
  console.log(p);
  let p1 = User.find({username:u}).exec();
  p1.then( (results) => {
    console.log(results);
    if (results.length >= 1) {

      let existingSalt = results[0].salt;
      let toHash = req.body.password + existingSalt;
      var hash = crypto.createHash('sha3-256');
      let data = hash.update(toHash, 'utf-8');
      let newHash = data.digest('hex');
      
      if (newHash == results[0].hash) {
        let id = cm.sessions.addOrUpdateSession(u);
        res.cookie("login", {username: u, sid: id}, {maxAge: 30000});
        res.end('SUCCESS ' + JSON.stringify(results));
      } else {
        res.end('password was incorrect');
      }
    } else {
      res.end('login failed');
    }
  });
  p1.catch( (error) => {
    res.end('login failed');
  });
});

// Start up the server to listen on port 80
const port = 80;
app.listen(port, () => { console.log('server has started: localhost:80'); });

